{ Knihovna funkci pro manipulaci se stringy

  Autor:  Gord
  Verze 1.3.1
    - (2.10.2013)zmena typu parametru funkce Base64ToBitmap (kvuli prechodu na novy GG)
    - (15.12.2013) oprava DecToHex pro nektera zaporna ciska vypisuje 0000000
    - (15.12.2013) fova funkce FetchFight()

  ----------------
  Funkce verze:  1.0.0
    StringCompress(str:string):string;
      - vyhaze mezery, prevede na mala pismena, prvni znak slova vzdy velky
      - ekvivalent FilenameFromText z knihovny RelToAbsPath

    RemoveInvalidPathChars(str:string):string;
      - odstrani znaky, ktere mohou delat potize ve jmenech souboru

    IntToStrLength(iNum,len:integer):string;
      - vypis Int na zadany pocet znaku s levostrannymi nulami

    Base64ToBitmap(str:string):TBitMap;
      - vytvoreni bitove mapy z Base64 stringu (uvolnit bitmapu musi volajici)

  Funkce verze 1.1.0
    StrToStrings(str,separator:string):TStrings;
      - rozdeli string podle oddelovace 'separator' a prevede do TStrings

    StringsToStr(a:sTStrings; separator:string):string;
      - prevede TStrings na string oddeleny 'separatorem'

    GetCurrentPluginVersion:string;
      - vraci cislo verze pluginu nastavene v *.meta.ini souboru proave spusteneho
        pluginu

  Funkce verze 1.2.0
    function HexCharToInt(HexChar: String): Integer;
    function DecToHex(iNum,len:integer):string;
    function HexToDec(sHex: String): Integer;

  Funkce verze 1.3.0
    function AutoWrapText(text:string;len:integer):string;

  Funkce verze 1.4.0 - prace s CSV souborem
    procdure LibCsvInit();
      - inicializace bufferu pro cteni radku a polozek z CSV souboru
    procedure LibCsvFree();
      - uvolni pamet, ...
    function LibCsvReadLine(fStream:TFileStream):string;
      - nacte ze streamu radek
    function LibCsvParseRow(row:string);
      - naplni LibCsvItems polozkami z radku CSV souboru
}

//odstraneni znaku, ktere mohou delat potize ve jmenech souboru
function RemoveInvalidPathChars(str:string):string;
begin
  Result:=RegexReplace('[~^/|;,*!?@&\"''<>=+{}]',str,'',false);
  //dvojtecka v ceste, pokud neni za pismenem disku, je chybna a mela by jit pryc
end;

//vyhaze mezery, prevede na mala pismena, prvni znak slova vzdy velky
function StringCompress(str:string):string;
var VelkePismenoZa:string;
    s:string;
    i,v:integer;
begin
  VelkePismenoZa:=' `~!?@%^&*+=:;,"<>''|/\';
  Result:='';
  v:=1;
  str:=LowerCase(str);
  for i:=1 to length(str) do
  begin
    s:=copy(str,i,1);
    if(pos(s,VelkePismenoZa)>0) then v:=1
    else begin
      if(v=1) then Result:=Result+UpperCase(s)
      else Result:=Result+s;
      v:=0;
    end;
  end;
  Result:=Result;
end;

//vypis Int na zadany pocet znaku s levostrannymi nulami
function IntToStrLength(iNum,len:integer):string;
begin
  Result:=IntToStr(iNum);
  if(length(Result)<len) then begin
    Result:=copy('0000000000000000000000000000000',1,len-length(Result))+Result;
  end;
end;

//vytvoreni bitove mapy z Base64 stringu (uvolnit bitmapu musi volajici)
function Base64ToBitmap(str:ansistring):TBitMap;
var s:ansistring;
    stream:TMemoryStream;
begin
  stream:=TMemoryStream.Create;
  try
    s:=DecodeBase64(str);
    stream.Write(s,Length(s));
    stream.Position:=0;
    Result:=TBitMap.Create();
    Result.LoadFromStream(stream);
  finally
    stream.free;
  end;
end;

function StrToStrings(str,separator:string):TStrings;
begin
  Result:=TStringList.Create();
  while (str<>'') do begin
    Result.Add(Fetch(str,separator));
  end;
end;

function StringsToStr(a:TStrings;separator:string):string;
var i:integer;
begin
  Result:='';
  for i:=0 to a.Count-1 do begin
    if(i>0) then Result:=Result+separator;
    Result:=Result+a[i];
  end;
end;

function GetCurrentPluginVersion:string;
begin
  Result:=GeoGetPackageVersion(ReplaceString(GEOGET_SCRIPTFULLNAME,'.ggp.pas','.meta.ini'));
end;

function HexCharToInt(HexChar: String): Integer;  {prevod jednoho Hex znaku na Int}
begin
  Result:=0;
  HexChar:=Uppercase(HexChar);
  if((HexChar>='0') and (HexChar<='9')) then Result:=Ord(HexChar[1])-48
  else if((HexChar>='A') and (HexChar<='Z')) then Result:=Ord(HexChar[1])-65+10;
end;

function HexToDec(sHex: String): Integer;   {prevod Hex na Dec}
var i:integer;
begin
  Result:=0;
  for i:=1 to Length(sHex) do begin
    Result:=Result+(HexCharToInt(copy(sHex,i,1))*Trunc(Power(16,Length(sHex)-i)));
  end;
end;

function DecToHex(iNum,len:integer):string;
var iByte:byte;
    s:string;
begin
  s:='0123456789ABCDEF';
  Result:='';
  while(iNum<>0) do begin
    iByte:=iNum and $0f;
    iNum:=iNum shr 4;
    Result:=s[iByte+1]+Result;
  end;
  if(length(Result)<len) then Result:=copy('00000000',1,len-length(Result))+Result
  else if(length(Result)>len) then Result:=copy(Result,1,len);
end;

function AutoWrapText(text:string;len:integer):string;
var reg:string;
begin
  //zalomi text na bilem znaku nejblize k zadane delce
  reg:='(.{0,'+IntToStr(len)+'}[^\s]*)\s?';
  Result:=RegexReplace(reg,trim(text),'$1'+CRLF,true);
  //Result:=RegexReplace(CRLF+'$',Result,'',true);
end;

var rLibCsv:record
    iBufferLength:integer;
    fStream:TFileStream;
    cDelimiter:char;   //oddelovac polozek, implicitne carka
    sRest:ansistring;   //nacteny zbytek - zacatek dalsiho radku
    //sBuffer2:ansistring; //
    aItems:TStringList; //parsovane polozky z radku
    end;
procedure LibCsvInit(fStream:TFileStream); //inicializace bufferu pro cteni radku a polozek z CSV souboru
begin
  rLibCsv.iBufferLength:=10000;
  rLibCsv.fStream:=fStream;
  rLibCsv.sRest:='';
  rLibCsv.aItems:=TStringList.Create();
  //(script enine to nepodporuje)
  //rLibCsv.aItems.DelimiterText:=',';
  //rLibCsv.aItems.QuoteChar:='"';
end;

procedure LibCsvFree; //uvolni pamet, ...
begin
  rLibCsv.aItems.Free();
  rLibCsv.aItems:=nil;
  rLibCsv.sRest:='';
  rLibCsv.fStream:=nil;
end;

function LibCsvParseRow(row:string):TStringList;//naplni LibCsv.aItems polozkami z radku CSV souboru a vrati je
begin
  rLibCsv.aItems.CommaText:=row;
  //nebo lepe (ale script enine to nepodporuje)
  //rLibCsv.aItems.DelimitedText:=row;
  Result:=rLibCsv.aItems;
end;

function LibCsvIsRowSeparator(sStr:ansistring; var bLFInItem:boolean):integer;
var iLen,nUvozovky:integer;
begin
  {
  Result:=Pos(#10,sStr);
  }
  //### tady by bylo potreba osetrit konce radku uprostred polozky, tedy
  //### mezi uvozovkami, ale tohle neni odladene a cteni gcm katalogu CWG
  //### v CwgList2 se zacykli (nebo je to tak pomale proti Pos() funkci,
  //### ze jsem nevydrzel cekat?
  iLen:=Length(sStr);
  nUvozovky:=0;
  bLfInItem:=false;
  for Result:=1 to iLen do begin
    if(sStr[Result]='"') then Inc(nUvozovky);
    if(sStr[Result]=#10) then begin
      //pokud je pred LF lichy pocet uvozovek, jsme uprostred polozky a neni to konec radku
      if((nUvozovky mod 2)=0) then exit;
      bLfInItem:=true;
    end;
  end;
  Result:=-1;
end;

function LibCsvReadLine:string; //nacte ze streamu radek
var sAnsi:ansistring;
    bEnd,bLfInclude:boolean;
    i:integer;
begin
  Result:='';
  bEnd:=rLibCsv.fStream.Position>=rLibCsv.fStream.Size;
//{$ifdef DEBUG_HELPER} LDH('LibCsvReadLine(): start, bEnd='+IntToStr(bEnd as integer)); {$endif}
  while(true) do begin
    //otestujeme, zda uz je nacten konec radku
    i:=LibCsvIsRowSeparator(rLibCsv.sRest,bLfInclude);
    if(i>0) then begin //pokud je nacten konec radku, vratime radek
      Result:=Trim(Copy(rLibCsv.sRest,1,i));
//{$ifdef DEBUG_HELPER} LDH('LibCsvReadLine(): Obsahuje nejaka polozka radku odradkovani? '+IntToStr(bLfInclude as integer)); {$endif}
//{$ifdef DEBUG_HELPER} LDH('LibCsvReadLine(): Vraceny radek: '+Result); {$endif}
      rLibCsv.sRest:=Copy(rLibCsv.sRest,i+1,Length(rLibCsv.sRest));
      SetLength(sAnsi,0);
      exit;
    end;
    if(bEnd) then begin
{$ifdef DEBUG_HELPER} LDH('LibCsvReadLine(): docten konec souboru, i='+IntToStr(i)
  +', sRest='+rLibCsv.sRest); {$endif}
      break;
    end;
    SetLength(sAnsi,rLibCsv.iBufferLength);
    i:=rLibCsv.fStream.Read(sAnsi,rLibCsv.iBufferLength);  //pokud jsme na konci, rovnou navrat
    if(i<>rLibCsv.iBufferLength) then begin
      SetLength(sAnsi,i); //zkratime na skutecne nacteny pocet znaku
      bEnd:=true;
    end;
    rLibCsv.sRest:=rLibCsv.sRest+sAnsi;
  end;
  //sem se dostaneme jen kdyz neni nacten konec radku na konci souboru
  Result:=rLibCsv.sRest;
  rLibCsv.sRest:='';
  SetLength(sAnsi,0);
end;

